@TestGroup(description: 'tags.builder')
library tags.test.builder;

import 'dart:html';
import 'package:tags/tags.dart';
import 'package:bench/bench.dart';
import 'package:unittest/unittest.dart';

Tag hasElementA = createTag((_) => new HasElementA().element);
class HasElementA extends HasElement {
  final Element element = span()..text = 'snarf';
}

Tag hasElementB = createTag((_) => new HasElementB().element);
class HasElementB extends HasElement {
  final Element element = 
      div(() {
        hasElementA();
      });
}

@Test()
void testCreateTagNoArgs() {
  Tag tag = createTag((_) {
    expect(buildHostNode, isNull);
    return new Element.tag('span');
  });
  expect(tag, const isInstanceOf<Tag>());
  expect(tag(), const isInstanceOf<SpanElement>());
}

@Test()
void testCreateTagWithArgs() {
  final o = { 'foo' : 'bar' };
  Tag tag = createTag((args) {
    expect(buildHostNode, isNull);
    expect(args, equals(o));
    return new Element.tag('span');
  });
  expect(tag(null, o), const isInstanceOf<SpanElement>());
}

@Test()
void testNestedHasElementCreationInBuildScope() {
  var e = div(() {
    hasElementB();
  });
  expect(e.children.length, equals(1));
  var b = e.children[0];
  expect(b, const isInstanceOf<DivElement>());    
  expect(b.children.length, equals(1));
  var a = b.children[0];
  expect(a, const isInstanceOf<SpanElement>());
  expect(a.text, equals('snarf'));
}

@Test()
void testAppend() {
  SpanElement s = new SpanElement();
  var e = div(() {
    append(s);
  });
  expect(e.children.length, equals(1));
  expect(e.children[0], equals(s));
}

@Test()
void testAppendText() {
  var e = div(() {
    text('snarf');
  });
  expect(e.children, isEmpty);
  expect(e.childNodes.length, equals(1));
  expect(e.childNodes[0].nodeType, equals(Node.TEXT_NODE));
  expect(e.childNodes[0], const isInstanceOf<Text>());
  expect(e.childNodes[0].text, equals('snarf'));  
}

@Test()
@ExpectError(isUnsupportedError)
void testAppendTextBuildHostNotElementThrows() {
  div(() {
    shadow(() {
      text('yo');
    });
  });
}

@Test()
void testShadow() {
  ShadowRoot s;
  var e = div(() {
    s = shadow(() {
      span();
    });
  });
  expect(e.children, isEmpty);
  expect(e.shadowRoot, equals(s));
  expect(s.children.length, equals(1));
  expect(s.children[0], const isInstanceOf<SpanElement>());
}

@Test()
@ExpectError(isUnsupportedError)
void testShadowBuildHostNotElementThrows() {
  div(() {
    shadow(() {
      shadow();
    });
  });
}

@Test()
void testChildren() {
  var e = div();
  children(e, () {
    span();
  });
  expect(e.children.length, equals(1));
  expect(e.children[0], const isInstanceOf<SpanElement>());
}
